@@ -16,7 +16,7 @@ use Test::Builder;
 	file_executable_ok file_not_executable_ok
 	file_mode_is file_mode_isnt
 	file_mode_has file_mode_hasnt
-	file_is_symlink_ok
+	file_is_symlink_ok 	file_is_not_symlink_ok
 	symlink_target_exists_ok symlink_target_is
 	symlink_target_dangles_ok
 	dir_exists_ok dir_contains_ok
@@ -30,7 +30,7 @@ use Test::Builder;
 	file_mtime_gt_ok file_mtime_lt_ok file_mtime_age_ok
 	);
 
-$VERSION = '1.443_03';
+$VERSION = '1.447_01';
 
 my $Test = Test::Builder->new();
 
@@ -81,7 +81,7 @@ sub _normalize {
 sub _win32 {
 	return 0 if $^O eq 'darwin';
 	return $ENV{PRETEND_TO_BE_WIN32} if defined $ENV{PRETEND_TO_BE_WIN32};
-	return $^O =~ m/Win/;
+	return $^O =~ m/Win/ || $^O eq 'msys';
 	}
 
 # returns true if symlinks can't exist
@@ -152,11 +152,11 @@ sub file_not_exists_ok {
 
 =item file_empty_ok( FILENAME [, NAME ] )
 
-Ok if the file exists and has empty size, not ok if the
-file does not exist or exists with non-zero size.
+Ok if the file exists and has empty size, not ok if the file does not
+exist or exists with non-zero size.
 
 Previously this tried to test any sort of file. Sometime in the future
-this will fail if the argument is not a plain file.
+this will fail if the argument is not a plain file or is a directory.
 
 =cut
 
@@ -173,6 +173,10 @@ sub file_empty_ok {
 		$Test->diag( "File [$filename] is not a plain file, which is deprecated for file_empty_ok" );
 		}
 
+	if( -d $filename ) {
+		$Test->diag( "File [$filename] is a directory, which is deprecated for file_empty_ok" );
+		}
+
 	my $ok = -z $filename;
 
 	if( $ok ) {
@@ -190,7 +194,7 @@ Ok if the file exists and has non-zero size, not ok if the file does
 not exist or exists with zero size.
 
 Previously this tried to test any sort of file. Sometime in the future
-this will fail if the argument is not a plain file.
+this will fail if the argument is not a plain file or is a directory.
 
 =cut
 
@@ -207,6 +211,10 @@ sub file_not_empty_ok {
 		$Test->diag( "File [$filename] is a directory, which is deprecated for file_not_empty_ok" );
 		}
 
+	if( -d $filename ) {
+		$Test->diag( "File [$filename] is a directory, which is deprecated for file_not_empty_ok" );
+		}
+
 	my $ok = not -z _;
 
 	if( $ok ) {
@@ -224,7 +232,7 @@ Ok if the file exists and has SIZE size in bytes (exactly), not ok if
 the file does not exist or exists with size other than SIZE.
 
 Previously this tried to test any sort of file. Sometime in the future
-this will fail if the argument is not a plain file.
+this will fail if the argument is not a plain file or is a directory.
 
 =cut
 
@@ -242,6 +250,10 @@ sub file_size_ok {
 		$Test->diag( "File [$filename] is a directory, which is deprecated for file_size_ok" );
 		}
 
+	if( -d $filename ) {
+		$Test->diag( "File [$filename] is a directory, which is deprecated for file_size_ok" );
+		}
+
 	my $ok = ( -s $filename ) == $expected;
 
 	if( $ok ) {
@@ -263,7 +275,7 @@ ok if the file does not exist or exists with size greater than MAX
 bytes.
 
 Previously this tried to test any sort of file. Sometime in the future
-this will fail if the argument is not a plain file.
+this will fail if the argument is not a plain file or is a directory.
 
 =cut
 
@@ -281,6 +293,10 @@ sub file_max_size_ok {
 		$Test->diag( "File [$filename] is a directory, which is deprecated for file_max_size_ok" );
 		}
 
+	if( -d $filename ) {
+		$Test->diag( "File [$filename] is a directory, which is deprecated for file_max_size_ok" );
+		}
+
 	my $ok = ( -s $filename ) <= $max;
 
 	if( $ok ) {
@@ -304,7 +320,7 @@ not ok if the file does not exist or exists with size less than MIN
 bytes.
 
 Previously this tried to test any sort of file. Sometime in the future
-this will fail if the argument is not a plain file.
+this will fail if the argument is not a plain file or is a directory.
 
 =cut
 
@@ -322,6 +338,10 @@ sub file_min_size_ok {
 		$Test->diag( "File [$filename] is a directory, which is deprecated for file_min_size_ok" );
 		}
 
+	if( -d $filename ) {
+		$Test->diag( "File [$filename] is a directory, which is deprecated for file_min_size_ok" );
+		}
+
 	my $ok = ( -s $filename ) >= $min;
 
 	if( $ok ) {
@@ -347,7 +367,7 @@ This function uses the current value of C<$/> as the line ending and
 counts the lines by reading them and counting how many it read.
 
 Previously this tried to test any sort of file. Sometime in the future
-this will fail if the argument is not a plain file.
+this will fail if the argument is not a plain file or is a directory.
 
 =cut
 
@@ -378,6 +398,14 @@ sub file_line_count_is {
 		shift || "$filename line count is $expected lines";
 		};
 
+	unless( ! -e $filename or -f $filename ) {
+		$Test->diag( "File [$filename] is not a plain file, which is deprecated for file_line_count_is" );
+		}
+
+	if( -d $filename ) {
+		$Test->diag( "File [$filename] is a directory, which is deprecated for file_line_count_is" );
+		}
+
 	unless( defined $expected && int( $expected ) == $expected ) {
 		no warnings 'uninitialized';
 		$Test->diag( "file_line_count_is expects a positive whole number for " .
@@ -420,7 +448,7 @@ This function uses the current value of C<$/> as the line ending and
 counts the lines by reading them and counting how many it read.
 
 Previously this tried to test any sort of file. Sometime in the future
-this will fail if the argument is not a plain file.
+this will fail if the argument is not a plain file or is a directory.
 
 =cut
 
@@ -432,6 +460,14 @@ sub file_line_count_isnt {
 		shift || "$filename line count is not $expected lines";
 		};
 
+	unless( ! -e $filename or -f $filename ) {
+		$Test->diag( "File [$filename] is not a plain file, which is deprecated for file_line_count_isnt" );
+		}
+
+	if( -d $filename ) {
+		$Test->diag( "File [$filename] is a directory, which is deprecated for file_line_count_isnt" );
+		}
+
 	unless( defined $expected && int( $expected ) == $expected ) {
 		no warnings 'uninitialized';
 		$Test->diag( "file_line_count_is expects a positive whole number for " .
@@ -473,7 +509,7 @@ This function uses the current value of C<$/> as the line ending and
 counts the lines by reading them and counting how many it read.
 
 Previously this tried to test any sort of file. Sometime in the future
-this will fail if the argument is not a plain file.
+this will fail if the argument is not a plain file or is a directory.
 
 =cut
 
@@ -482,6 +518,14 @@ sub file_line_count_between {
 	my $min      = shift;
 	my $max      = shift;
 
+	unless( ! -e $filename or -f $filename ) {
+		$Test->diag( "File [$filename] is not a plain file, which is deprecated for file_line_count_between" );
+		}
+
+	if( -d $filename ) {
+		$Test->diag( "File [$filename] is a directory, which is deprecated for file_line_count_between" );
+		}
+
 	my $name = do {
 		no warnings 'uninitialized';
 		shift || "$filename line count is between [$min] and [$max] lines";
@@ -657,7 +701,7 @@ sub _file_contains {
 		return $Test->ok(0, $name);
 		}
 
-	unless( -f $filename ) {
+	if( -d $filename ) {
 		my $caller = ( caller(0) )[3];
 		$Test->diag( "File [$filename] is a directory, which is deprecated for $caller" );
 		}
@@ -1007,7 +1051,7 @@ sub file_mode_hasnt {
 
 Ok if FILENAME is a symlink, even if it points to a non-existent
 file. This test automatically skips if the operating system does
-not support symlinks. If the file does not exist, the test fails.
+not support symlinks.
 
 =cut
 
@@ -1030,6 +1074,38 @@ sub file_is_symlink_ok {
 		}
 	}
 
+=item file_is_not_symlink_ok( FILENAME [, NAME ] )
+
+Ok if FILENAME is a not symlink. This test automatically skips if the
+operating system does not support symlinks. If the file does not
+exist, the test fails.
+
+=cut
+
+sub file_is_not_symlink_ok {
+	if( _no_symlinks_here() ) {
+		$Test->skip(
+			"file_is_symlink_ok doesn't work on systems without symlinks!" );
+		return;
+		}
+
+	my $file = shift;
+	my $name = shift || "$file is not a symlink";
+
+	unless( -e $file ) {
+		$Test->diag( "File [$file] does not exist!" );
+		return $Test->ok(0, $name);
+		}
+
+	if( ! -l $file ) {
+		$Test->ok(1, $name)
+		}
+	else {
+		$Test->diag( "File [$file] is a symlink!" );
+		$Test->ok(0, $name);
+		}
+	}
+
 =item symlink_target_exists_ok( SYMLINK [, TARGET] [, NAME ] )
 
 Ok if FILENAME is a symlink and it points to a existing file. With the
@@ -1518,36 +1594,93 @@ sub group_isnt {
 	}
 
 sub _get_uid {
-	my $owner = shift;
-	my $owner_uid;
+	my $arg = shift;
 
-	if ($owner =~ /^\d+/) {
-		$owner_uid = $owner;
-		$owner = ( getpwuid $owner )[0];
-		}
-	else {
-		$owner_uid = (getpwnam($owner))[2];
-		}
+	# the name might be numeric (why would you do that?), so we need
+	# to figure out which of several possibilities we have. And, 0 means
+	# root, so we have to be very careful with the values.
 
-	$owner_uid;
+	# maybe the argument is a UID. First, it has to be numeric. If it's
+	# a UID, we'll get the same UID back. But, if we get back a value
+	# that doesn't mean that we are done. There might be a name with
+	# the same value.
+	#
+	# Don't use this value in comparisons! An undef could be turned
+	# into zero!
+	my $from_uid = (getpwuid($arg))[2] if $arg =~ /\A[0-9]+\z/;
+
+	# Now try the argument as a name. If it's a name, then we'll get
+	# back a UID. Maybe we get back nothing.
+	my $from_nam = (getpwnam($arg))[2];
+
+	return do {
+		# first case, we got back nothing from getpwnam but did get
+		# something from getpwuid. The arg is not a name and is a
+		# UID.
+		   if( defined $from_uid and not defined $from_nam ) { $arg }
+		# second case, we got back nothing from getpwuid but did get
+		# something from getpwnam. The arg is a name and is not a
+		# UID.
+		elsif( not defined $from_uid and defined $from_nam ) { $from_nam }
+		# Now, what happens if neither are defined? The argument does
+		# not correspond to a name or GID on the system. Since no such
+		# user exists, we return undef.
+		elsif( not defined $from_uid and not defined $from_nam ) { undef }
+		# But what if they are both defined? The argument could represent
+		# a UID and a name, and those could be different users! In this
+		# case, we'll choose the original argument. That might be wrong,
+		# so the best we can do is a warning.
+		else {
+			carp( "Found both a UID or name for <$arg>. Guessing the UID is <$arg>." );
+			$arg
+			}
+		};
 	}
 
 sub _get_gid {
-	my $group = shift;
-	my $group_uid;
+	my $arg = shift;
 
-	if ($group =~ /^\d+/) {
-		$group_uid = $group;
-		$group = ( getgrgid $group )[0];
-		}
-	else {
-		$group_uid = (getgrnam($group))[2];
-		}
+	# the name might be numeric (why would you do that?), so we need
+	# to figure out which of several possibilities we have. And, 0 means
+	# root, so we have to be very careful with the values.
 
-	$group_uid;
+	# maybe the argument is a GID. First, it has to be numeric. If it's
+	# a GID, we'll get the same GID back. But, if we get back a value
+	# that doesn't mean that we are done. There might be a name with
+	# the same value.
+	#
+	# Don't use this value in comparisons! An undef could be turned
+	# into zero!
+	my $from_gid = (getgrgid($arg))[2] if $arg =~ /\A[0-9]+\z/;
+
+	# Now try the argument as a name. If it's a name, then we'll get
+	# back a GID. Maybe we get back nothing.
+	my $from_nam = (getgrnam($arg))[2];
+
+	return do {
+		# first case, we got back nothing from getgrnam but did get
+		# something from getpwuid. The arg is not a name and is a
+		# GID.
+		   if( defined $from_gid and not defined $from_nam ) { $arg }
+		# second case, we got back nothing from getgrgid but did get
+		# something from getgrnam. The arg is a name and is not a
+		# GID.
+		elsif( not defined $from_gid and defined $from_nam ) { $from_nam }
+		# Now, what happens if neither are defined? The argument does
+		# not correspond to a name or GID on the system. Since no such
+		# user exists, we return undef.
+		elsif( not defined $from_gid and not defined $from_nam ) { undef }
+		# But what if they are both defined? The argument could represent
+		# a GID and a name, and those could be different users! In this
+		# case, we'll choose the original argument. That might be wrong,
+		# so the best we can do is a warning.
+		else {
+			carp( "Found both a GID or name for <$arg>. Guessing the GID is <$arg>." );
+			$arg;
+			}
+		};
 	}
 
-
 =item file_mtime_age_ok( FILE [, WITHIN_SECONDS ] [, NAME ] )
 
 Ok if FILE's modified time is WITHIN_SECONDS inclusive of the system's current time.
@@ -1558,7 +1691,7 @@ returns failure. If stat() fails, the test is skipped.
 
 sub file_mtime_age_ok {
 	my $filename    = shift;
-	my $within_secs = int shift || 0;
+	my $within_secs = shift || 0;
 	my $name        = shift || "$filename mtime within $within_secs seconds of current time";
 
 	my $time        = time();
@@ -1670,13 +1803,14 @@ sub _stat_file {
 L<Test::Builder>,
 L<Test::More>
 
-If you are using the new C<Test2> stuff, see L<Test2::Tool::File>.
+If you are using the new C<Test2> stuff, see Test2::Tools::File
+(https://github.com/torbjorn/Test2-Tools-File).
 
 =head1 SOURCE AVAILABILITY
 
 This module is in Github:
 
-	git://github.com/briandfoy/test-file.git
+	https://github.com/briandfoy/test-file.git
 
 =head1 AUTHOR
 
@@ -1705,7 +1839,7 @@ working together to align our interfaces.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2002-2020, brian d foy <bdfoy@cpan.org>. All rights reserved.
+Copyright © 2002-2021, brian d foy <bdfoy@cpan.org>. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the Artistic License 2.0
